#include <PropButton.h>
#include "KyloBlade.h"

// Button pin
#define BUTTON_PIN      1

// Blade pixels quantities
#define BLADE_PIXELS	  144
#define QUILLION_PIXELS	32

// Blade & effects colors
#define BLADE_COLOR		COLOR(255,0,0)
#define SPARKS_COLOR	COLOR(255,255,255)
#define CLASH_COLOR		COLOR(255,255,255)
#define BLASTER_COLOR	COLOR(255,255,255)

// Effect duration
#define RETRACTION_DURATION 750
#define CLASH_DURATION      250
#define BLASTER_DURATION    500
#define BLASTER_SIZE        15

// Delay in ms to lit quillions on ignition
#define QUILLIONS_DELAY	750

// Lightsaber states
#define STATE_OFF       0
#define STATE_POWER_ON  1
#define STATE_IDLE      2
#define STATE_POWER_OFF 3
#define STATE_HIT       4
#define STATE_BLASTER   5
#define STATE_WAIT_OFF  6

// Five minutes before entering low power
#define TIME_BEFORE_LOW_POWER 300000

// Our push button with LED
PropButton OnOffButton;

// The current state
uint8_t state = STATE_OFF;

// A booleans to signal a hit event
volatile bool on_hit = false;

// This variable holds the time (in milliseconds)
// since the program is in the STATE_OFF state.
uint32_t time_off;

// Our Kylo blade
KyloBlade blade;

void setup()
{
  // Initialize motion sensor
  Motion.begin();
  Motion.configPulse(AxisAll, 3.78f, 500, 100, MotionInterrupt1 );
  Motion.attachInterrupt(MotionInterrupt1, MotionPulse);

  // Initialize blade
  blade.begin(BLADE_PIXELS, QUILLION_PIXELS, WS2812B);
  blade.setColors(BLADE_COLOR, SPARKS_COLOR);
  
  // Initialize On/Off button
  OnOffButton.begin(BUTTON_PIN, ButtonActiveLow);

  // Configure the OnOffButton to declare a long press after 1000ms
  OnOffButton.setLongPressTime(1000);
  
  state = STATE_OFF;

  // We entered in the STATE_OFF state
  time_off = millis();
}

void loop()
{
  ButtonEvent button_event;
  blade.poll();

  // Check the current state
  switch(state)
  {	
    case STATE_OFF:
      // If On/Off button is pressed
      if (OnOffButton.pressed())
      {
        // Wait for the user to release the OnOffButton
        while (OnOffButton.pressed());

        // Go to STATE_POWER_ON
        state = STATE_POWER_ON;
      } else {
        // Check if we have to go into low-power
        if (millis() - time_off >= TIME_BEFORE_LOW_POWER)
        {
          // Enter low power mode. Wait for a FALLING edge
          // on the button pin.
          enterLowPowerMode(BUTTON_PIN, FALLING);
        }
      }
      break;

    case STATE_POWER_ON:

	    // Turn on the blade
	    blade.ignition(QUILLIONS_DELAY);

      // Ignore any motion detected previous to STATE_IDLE
      on_hit = false;

      // Clear any OnOffButton queued events
      OnOffButton.resetEvents();

      // Go to STATE_IDLE
      state = STATE_IDLE;
      break;

    case STATE_IDLE:
      // This is the main state.
      button_event = OnOffButton.getEvent();

      // If On/Off button is long-pressed, go to STATE_POWER_OFF.
      if (button_event == ButtonLongPressed)
      {
        state = STATE_POWER_OFF;
        break;
      }

      // If OnOffButton is pressed and released, go to STATE_BLASTER.
      if (button_event == ButtonShortPressAndRelease)
      {
        state = STATE_BLASTER;
        break;
      }

      // Check hits
      if (on_hit)
      {
        on_hit = 0;
        state = STATE_HIT;
        break;
      }
      break;

    case STATE_HIT:
      blade.clash(CLASH_COLOR, CLASH_DURATION);
      state = STATE_IDLE;
      break;

    case STATE_BLASTER:
      blade.blaster(BLASTER_COLOR, BLASTER_DURATION, BLASTER_SIZE);
      state = STATE_IDLE;
      break;

    case STATE_POWER_OFF:
      blade.retraction(RETRACTION_DURATION);
      state = STATE_WAIT_OFF;
      break;
	  
	case STATE_WAIT_OFF:
	  if (!blade.isOn())
	    state = STATE_OFF;
	  break;
  }
}

// Motion pulse interrupt (hit)
void MotionPulse()
{
  on_hit = true;
}
